function varargout = waterfall(u,varargin)
%WATERFALL Waterfall plot for quasimatrices.
%
%  WATERFALL(U) or WATERFALL(U,T) where LENGTH(T) = MIN(SIZE(U))
%
%  See http://www.maths.ox.ac.uk/chebfun for chebfun information.

%  Copyright 2002-2009 by The Chebfun Team. 
%  Last commit: $Author: hale $: $Rev: 1136 $:
%  $Date: 2010-06-09 15:16:24 +0100 (Wed, 09 Jun 2010) $:

% numpts = chebfunpref('plot_numpts');

numpts = 0; numptsmax = inf; numptsmin = 200;
simple = 0;

k = 1;
while k <= numel(varargin)
    if strcmpi(varargin{k},'numpts')
        numpts = varargin{k+1};
        varargin(k:k+1) = [];
    elseif strcmpi(varargin{k},'simple')
        varargin(k) = [];
        simple = true;
    else
        k = k+1;
    end
end
nargin = length(varargin)+1;

trans = u(:,1).trans;
if trans
    u = u.';
end
n = min(size(u,2));
t = 1:n;

if nargin > 1 && isnumeric(varargin{1}) && length(varargin{1}) == size(u,2)
    t = varargin{1}; t = t(:).';
    varargin = {varargin{2:end}};
end

if length(t)~=n
    error('CHEBFUN:waterfall:szet', ...
        'Length of T should equal the number of quasimatrices in U');
end

if ~isreal(u) || ~all(isreal(t))
    warning('CHEBFUN:waterfall:imaginary',...
        'Imaginary parts of complex T and/or U arguments ignored');
    u = real(u); t = real(t);
end    

if simple, 
    if numpts == 0
        numpts = max(chebfunpref('plot_numpts')/10,10);
    end
%     varargin = [varargin, {'numpts',numpts}]; 
%     x = chebfun('x',domain(u));
%     ish = ishold; h = {};
%     if u(1).trans, u = u.'; end
%     for k = 1:numel(u)
%         tk = t(k) + 0*x;
%         h{k} = plot3(x,tk,u(:,k),varargin{:}); hold on
%     end
%     if ~ish, hold off, end
%     if nargout > 0, varargout{1} = h; end
% %     return
end

% Sort out number of points to use in plot
if numpts == 0
    for k = 1:numel(u)
        numpts = max(numpts,length(u(k)));
    end
    numpts = max(min(numpts,numptsmax),numptsmin);
end
numpts = ceil(numpts);

% get the data
[data ignored data3] = plotdata([],u,[],numpts);
uu = data{2:end};
xx = repmat(data{1},1,n);
tt = repmat(t,length(xx(:,1)),1);

% mask the NaNs
mm = find(isnan(uu));
uu(mm) = uu(mm+1);

minuu = min(uu(:));


uu(1) = uu(1)+eps;

if simple
    ish = ishold;
    mesh(xx.',tt.',uu.','edgecolor','none'); hold on,
    % Intersperse with NaNs
    xx = repmat(xx,1,4);
    mid = repmat((t(1:end-1)+t(2:end))/2,size(tt,1),1);
    tt = reshape([tt ; tt+eps ; [mid NaN*tt(:,end)] ; [tt(:,2:end)-eps NaN*tt(:,end)]],size(xx));
%     uu1 = reshape([uu ; minuu + 0*uu ; NaN*uu ; minuu + 0*[uu(:,2:end) uu(:,end)]],size(xx));
%     hold off,  mesh(xx.',tt.',uu1.','edgecolor','none'); hold on
    uu = reshape([uu ; uu ; NaN*uu ; [uu(:,2:end) uu(:,end)]],size(xx));
    mesh(xx.',tt.',uu.',varargin{:});
    if ~ish, hold off; end
    
    % old way (using plot3)
%     varargin = [varargin, {'numpts',numpts}]; 
%     x = chebfun('x',domain(u));
%     ish = ishold; h = {};
%     if u(1).trans, u = u.'; end
%     for k = 1:numel(u)
%         tk = t(k) + 0*x;
%         h{k} = plot3(x,tk,u(:,k),varargin{:}); hold on
%     end
%     if ~ish, hold off, end
%     if nargout > 0, varargout{1} = h; end
%     return
end

% plot the waterfall
if ~simple
    h = waterfall(xx.',tt.',uu.',varargin{:});
end

% hide the jumps
if ~trans
    x = []; y = []; z = [];
    for k = 1:2:2*n
        x = [x ; data3{k}];
        y = [y ; t((k+1)/2)*ones(length(data3{k}),1)];
        z = [z ; data3{k+1}];
    end
    if ~all(isnan(z))
        x([1 end]) = [];
        y([1 end]) = [];
        z([1 end]) = [];
        ish = ishold;    hold on
        plot3(x,y,z,'w');
        if ~ish, hold off; end
    end
end 

u = reshape(uu,numel(uu),1);
umin = min(u); umax = max(u);
if umin - umax == 0
    zl = get(gca,'zlim')
    if abs(diff(zl)) < 1e-6
        umin+1e-2*[-1 1]
        set(gca,'zlim',umin+1e-2*[-1 1]);
    end
end

if nargout > 0
    varargout{1} = h;
end

